@@ -110,7 +110,7 @@ fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
         Vec3::new(margin, margin, 0.),
     ];
     // Build a sequence from an iterator over a Tweenable (here, a Tween<Transform>)
-    let seq = Sequence::new(dests.windows(2).map(|pair| {
+    let seq = Sequence::new(dests.windows(2).enumerate().map(|(index, pair)| {
         Tween::new(
             EaseFunction::QuadraticInOut,
             TweeningType::Once,
@@ -120,6 +120,7 @@ fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
                 end: pair[1] - center,
             },
         )
+        .with_completed_event(true, index as u64) // Get an event after each segment
     }));
 
     commands
@@ -144,7 +145,8 @@ fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
             start: Vec3::new(-200., 100., 0.),
             end: Vec3::new(200., 100., 0.),
         },
-    );
+    )
+    .with_completed_event(true, 99); // Get an event once move completed
     let tween_rotate = Tween::new(
         EaseFunction::QuadraticInOut,
         TweeningType::Once,
@@ -193,6 +195,7 @@ fn update_text(
     )>,
     query_anim_red: Query<&Animator<Transform>, With<RedSprite>>,
     query_anim_blue: Query<&Animator<Transform>, With<BlueSprite>>,
+    mut query_event: EventReader<TweenCompleted>,
 ) {
     let anim_red = query_anim_red.single();
     let tween_red = anim_red.tweenable().unwrap();
@@ -213,4 +216,11 @@ fn update_text(
         let mut blue_text = q1.single_mut();
         blue_text.sections[1].value = format!("{:5.1}%", progress_blue * 100.).to_string();
     }
+
+    for ev in query_event.iter() {
+        println!(
+            "Event: TweenCompleted entity={:?} user_data={}",
+            ev.entity, ev.user_data
+        );
+    }
 }
